'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.findProjectRoot = findProjectRoot;
exports.getLocalConfig = getLocalConfig;
exports.parseGitUrl = parseGitUrl;
exports.getLocalDepPaths = getLocalDepPaths;
exports.getLocalRemovePaths = getLocalRemovePaths;
exports.toString = toString;
exports.toObject = toObject;
exports.getPageOptions = getPageOptions;
exports.getAppOptions = getAppOptions;
exports.wrapAppJsOptions = wrapAppJsOptions;
exports.wrapPageOptions = wrapPageOptions;
exports.unShiftCode = unShiftCode;
exports.multiMinmatch = multiMinmatch;
exports.fetchLatestWeshopConfig = fetchLatestWeshopConfig;
exports.sparseCheckout = sparseCheckout;
exports.isEmptyDirectory = isEmptyDirectory;
exports.hasInited = hasInited;
exports.getDirBranch = getDirBranch;
exports.isMac = isMac;
exports.createTmpDir = createTmpDir;
exports.getProjectNameFromGitUrl = getProjectNameFromGitUrl;
exports.getWorkDirs = getWorkDirs;
exports.addGitIgnore = addGitIgnore;
exports.isMainPackage = isMainPackage;
exports.gitChildProcessParser = gitChildProcessParser;
exports.logStepCaption = logStepCaption;
exports.isCommon = isCommon;
exports.getGitLastCommit = getGitLastCommit;
exports.getGitCurrentBranch = getGitCurrentBranch;
exports.getGitLastAuthor = getGitLastAuthor;
exports.exportAssembleReport = exportAssembleReport;
exports.getParam = getParam;
exports.buildTag = buildTag;

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _crossSpawn = require('cross-spawn');

var _crossSpawn2 = _interopRequireDefault(_crossSpawn);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _constant = require('./constant');

var _tosource = require('tosource');

var _tosource2 = _interopRequireDefault(_tosource);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _minimatch = require('minimatch');

var _minimatch2 = _interopRequireDefault(_minimatch);

var _md = require('md5');

var _md2 = _interopRequireDefault(_md);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _child_process = require('child_process');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 寻找项目根目录，以weshop.config.json的存在为判定标准，直到追溯到系统根目录
 * @returns {String}
 */
function findProjectRoot() {
  var root = process.cwd();
  var isMac = _os2.default.type().toLowerCase() === 'darwin';

  if (!global.PROJECT_ROOT) {
    while (!_fs2.default.existsSync(_path2.default.resolve(root, _constant.WESHOP_CONFIG_FILE_NAME))) {
      if (isMac && root === '/' || !isMac && /^[a-zA-Z]:\\$/.test(root)) {
        throw new ReferenceError('\u627E\u4E0D\u5230\u6587\u4EF6 ' + _constant.WESHOP_CONFIG_FILE_NAME + '\uFF0C\u8FD9\u4E0D\u662F\u4E00\u4E2Aweshop\u5DE5\u7A0B\u76EE\u5F55');
      }
      root = _path2.default.resolve(root, '..');
    }
    global.PROJECT_ROOT = root;
  }
  return global.PROJECT_ROOT;
}

/**
 * 获取weshop.config.json配置对象
 * @returns {Object}
 */
function getLocalConfig() {
  var PROJECT_ROOT = findProjectRoot();
  var WESHOP_CONFIG_FILE_PATH = _path2.default.join(PROJECT_ROOT, _constant.WESHOP_CONFIG_FILE_NAME);

  var config = _fs2.default.existsSync(WESHOP_CONFIG_FILE_PATH) ? _fsExtra2.default.readJsonSync(WESHOP_CONFIG_FILE_PATH) : null;

  if (!config) {
    throw new ReferenceError('\u627E\u4E0D\u5230\u6587\u4EF6 ' + _constant.WESHOP_CONFIG_FILE_NAME);
  }

  return config;
}

/**
 * 解析Git Url，分离为仓库地址和分支
 * @param url
 * @returns {{repository: *|string, branch: *|string|string}}
 */
function parseGitUrl(url) {
  var parts = url.split('#');
  var repository = parts[0];
  var branch = parts[1] || 'master';
  var packageName = repository.split('/').pop().split('.')[0];
  return {
    repository: repository,
    branch: branch,
    packageName: packageName
  };
}

/**
 * 获取所有依赖复制到本地的路径绝对地址
 * @param {Function} [handler] 路径处理函数，该函数会遍历每一个依赖路径绝对地址，并返回处理后的地址，多用于路径形式的修改
 * @returns {Array}
 */
function getLocalDepPaths(handler) {
  var PROJECT_ROOT = findProjectRoot();
  var WESHOP_CONFIG = getLocalConfig();
  var dependencies = WESHOP_CONFIG.dependencies;
  var localPaths = [];
  handler = handler || function (p) {
    return p;
  };
  Array.isArray(dependencies) && dependencies.forEach(function (dependence) {
    var localBase = dependence.path || '.';
    var remotePaths = dependence.remotePaths;
    var copyRootDir = dependence.copyRootDir;
    if (remotePaths) {
      if (Array.isArray(remotePaths)) {
        remotePaths.forEach(function (remotePath) {
          if (copyRootDir) {
            remotePath = remotePath.replace(copyRootDir + '/', '');
          }
          localPaths.push(handler(_path2.default.join(PROJECT_ROOT, localBase, remotePath)));
        });
      } else {
        throw new TypeError('remotePaths格式错误');
      }
    } else {
      var repoName = getProjectNameFromGitUrl(dependence.repo);
      localPaths.push(handler(_path2.default.join(PROJECT_ROOT, localBase, repoName)));
    }
  });
  return localPaths;
}

function getLocalRemovePaths(handler) {
  var PROJECT_ROOT = findProjectRoot();
  var WESHOP_CONFIG = getLocalConfig();
  var removeItems = WESHOP_CONFIG.removeItems;
  var localPaths = [];
  handler = handler || function (p) {
    return p;
  };
  Array.isArray(removeItems) && removeItems.forEach(function (dependence) {
    var localBase = dependence.path || '.';
    var itemPaths = dependence.itemPaths;
    if (itemPaths && Array.isArray(itemPaths)) {
      itemPaths.forEach(function (remotePath) {
        localPaths.push(handler(_path2.default.join(PROJECT_ROOT, localBase, remotePath)));
      });
    } else {
      throw new TypeError('itemPaths格式错误');
    }
  });
  return localPaths;
}

/**
 * 将非标准JSON对象转化为字面字符串
 * @param {Object} o 待转换的对象
 * @returns {String}
 */
function toString(o) {
  return JSON.stringify(o, function (k, v) {
    if (typeof v === 'function') {
      return v.toString();
    } else {
      return v;
    }
  }, _constant.INDENT);
}

/**
 * 将对象字面字符串转换为非标准JSON对象
 * @param {String} s 待转换的字符串
 * @returns {Object}
 */
function toObject(s) {
  var es6FuncReg = /^(\(?[^\(\)]+\)?)\s=>/;
  return JSON.parse(s, function (k, v) {
    var es6MethodReg = new RegExp('^' + k + '\\s*\\([^)]*\\)');
    if (typeof v === 'string' && (v.indexOf('function') === 0 || es6MethodReg.test(v) || es6FuncReg.test(v))) {

      if (es6MethodReg.test(v)) {
        v = 'function ' + v;
      }
      var functionTemplate = '(' + v + ')';
      return eval(functionTemplate);
    } else {
      return v;
    }
  });
}

/**
 * 获取页面声明的构造参数对象
 * @param {String} pth 页面文件的路径
 * @returns {Object}
 */
function getPageOptions(pth, base) {
  var _getWorkDirs = getWorkDirs(),
      DIST_DIR = _getWorkDirs.DIST_DIR;

  base = base || DIST_DIR;

  function Page(o) {
    return o;
  }

  if (!_fs2.default.existsSync(pth)) {
    throw new ReferenceError('找不到文件'.red + ' ' + _path2.default.relative(base, pth).blue);
  }
  var code = _fs2.default.readFileSync(pth).toString();

  //console.log(code.match(PATTERNS.PAGE_CONSTRUCTOR));
  code = code.match(_constant.PATTERNS.PAGE_CONSTRUCTOR);
  if (!code) {
    throw new TypeError(('\u9875\u9762 ' + _path2.default.relative(base, pth) + ' \u672A\u8C03\u7528 Page \u6784\u9020\u51FD\u6570').red);
  }
  code = code[0];
  try {
    return toObject(toString(eval(code)));
  } catch (e) {
    var name = /([^\s]+)\sis not defined/.exec(e.message);
    if (!name) {
      throw new EvalError(('\n        \u9875\u9762 ' + _path2.default.relative(base, pth) + ' \u5305\u542B\u672A\u77E5\u89C4\u8303\u9519\u8BEF\n        ' + e.message + '\n      ').red);
    }
    name = name[1];
    throw new EvalError(('\u9875\u9762 ' + _path2.default.relative(base, pth) + ' \u7684Page\u53C2\u6570data\u4E2D\u4F7F\u7528\u4E86\u5916\u90E8\u5B9A\u4E49\u53D8\u91CF ' + name + '\uFF0C\u4E0D\u7B26\u5408\u9879\u76EE\u89C4\u8303').red);
  }
}

/**
 * 获取App声明的构造参数对象
 * @param {string} pth App声明文件路径
 * @returns {object}
 */
function getAppOptions(pth) {
  var _getWorkDirs2 = getWorkDirs(),
      DIST_DIR = _getWorkDirs2.DIST_DIR;

  if (!_fs2.default.existsSync(pth)) {
    throw new ReferenceError('找不到文件'.red + ' ' + _path2.default.relative(DIST_DIR, pth).blue);
  }
  var code = _fs2.default.readFileSync(pth).toString();
  try {
    code = code.match(_constant.PATTERNS.APP_OPTIONS)[0];
    return toObject(toString(eval(code)));
  } catch (e) {
    throw new EvalError((pth + '\u6587\u4EF6\u4E0D\u7B26\u5408\u89C4\u8303').red.red);
  }
}

/**
 * 使用新的构造参数对象生成新的app.js文件内容
 * @param {string} code 原文件内容
 * @param {object} options 新的构造参数对象
 * @returns {string}
 */
function wrapAppJsOptions(code, options) {
  return code.replace(/module\.exports\s*=\s*\{[\s\S]*\};?/g, 'module.exports=' + (0, _tosource2.default)(options) + ';');
}

/**
 * 使用新的构造参数对象生成新的页面声明文件内容
 * @param {string} code 原文件内容
 * @param {object} options 新的构造参数对象
 * @returns {string}
 */
function wrapPageOptions(code, options) {
  return code.replace(/(Page\s*\()\s*\{[\s\S]+\}\s*(\))/g, function (m, h, t) {
    return '' + h + (0, _tosource2.default)(options) + t;
  });
}

/**
 * 在原JS文件前部插如新内容，会自动补齐"use strict";
 * @param {string} code 原文件内容
 * @param {string} content 将要插入的内容
 * @returns {string}
 */
function unShiftCode(code, content) {
  content = content.trim().replace(/['"]use strict['"]\s*;?/, '');
  return '"use strict";' + code + content;
}

/**
 * 多重匹配，模仿gulp-filter的过滤数组
 * @param {string} pth 匹配验证的路径
 * @param {array} matches 匹配数组
 * @returns {boolean}
 */
function multiMinmatch(pth, matches) {
  if (!Array.isArray(matches)) {
    matches = [matches];
  }
  var matched = false;
  matches.forEach(function (match) {
    if ((0, _minimatch2.default)(pth, match)) {
      matched = true;
      return false;
    }
  });
  return matched;
}

/**
 * 从主项目获取最新的weshop.config.json
 */
function fetchLatestWeshopConfig(destPath) {
  var WESHOP_CONFIG = getLocalConfig();
  var mainPackage = WESHOP_CONFIG.mainPackage;

  console.log(('\u83B7\u53D6\u6700\u65B0' + _constant.WESHOP_CONFIG_FILE_NAME + '\u6587\u4EF6').green);
  sparseCheckout(mainPackage, ['weshop.config.json'], '', destPath);
  console.log(mainPackage);
}

/**
 * 稀疏拉取
 * @param {string} url 拉取代码的目标URL，支持#分支格式
 * @param {array} paths 要拉取的远程路径
 * @param {string} dest 本地放置代码的目录
 */
function sparseCheckout(url, paths) {
  var copyRootDir = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '.';
  var dest = arguments[3];
  var callback = arguments[4];

  var PROJECT_ROOT = findProjectRoot();

  var _parseGitUrl = parseGitUrl(url),
      repository = _parseGitUrl.repository,
      branch = _parseGitUrl.branch;

  var tmp = createTmpDir();
  dest = dest || PROJECT_ROOT;
  process.chdir(tmp);
  _crossSpawn2.default.sync('git', ['init'], {
    shell: true,
    stdio: 'inherit'
  });
  _crossSpawn2.default.sync('git', ['remote', 'add', 'origin', repository], {
    shell: true,
    stdio: 'inherit'
  });
  _crossSpawn2.default.sync('git', ['config', 'core.sparsecheckout', true], {
    shell: true,
    stdio: 'inherit'
  });
  paths.forEach(function (p) {
    _crossSpawn2.default.sync('echo', [p.replace(/\\/g, '\/'), '>>', ['.git', 'info', 'sparse-checkout'].join(_constant.SEP)], {
      shell: true,
      stdio: 'inherit'
    });
  });
  console.log(('\u83B7\u53D6 ' + repository).green);
  gitChildProcessParser(['pull', 'origin', branch], { shell: true });

  callback && callback(tmp);

  _fsExtra2.default.removeSync('.git');

  // 切换到原目录，进行复制（先删除后复制）
  process.chdir(dest);
  console.log('\u590D\u5236\u6587\u4EF6\u5230\u6307\u5B9A\u76EE\u5F55\uFF1A'.green, dest.blue);
  paths.forEach(function (p) {
    if (_fs2.default.existsSync(_path2.default.join(dest, p)) && !_path2.default.extname(p)) {
      console.log('\u5220\u9664\u672C\u5730\u76EE\u5F55'.green, p.blue);
      _fsExtra2.default.removeSync(_path2.default.join(dest, p));
    }
  });

  _fsExtra2.default.copySync(_path2.default.join(tmp, copyRootDir), dest + _constant.SEP);
  _fsExtra2.default.removeSync(tmp);
}

/**
 * 判断一个目录是否是空目录，包括隐藏文件
 * @param {string} dir 要判断的目录
 * @returns {boolean}
 */
function isEmptyDirectory(dir) {
  var excludes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  try {
    var dirList = _fs2.default.readdirSync(dir) || [];
    if (excludes.length) {
      dirList = dirList.reduce(function (all, item) {
        if (excludes.includes(item)) {
          return all;
        } else {
          return all.concat(item);
        }
      }, []);
    }
    return dirList.length === 0;
  } catch (e) {
    return false;
  }
}

/**
 * 判断当前目录是否已经初始化
 * @returns {boolean}
 */
function hasInited() {
  var _getWorkDirs3 = getWorkDirs(),
      WESHOP_WORK_DIR = _getWorkDirs3.WESHOP_WORK_DIR,
      WESHOP_DIRS = _getWorkDirs3.WESHOP_DIRS;

  var done = _fs2.default.existsSync(WESHOP_WORK_DIR);
  if (done) {
    WESHOP_DIRS.forEach(function (dir) {
      if (!_fs2.default.existsSync(dir)) {
        done = false;
      }
    });
  }
  return done;
}

/**
 * 获取当前文件夹的 git branch
 * @returns {string}
 */
function getDirBranch(dir) {
  if (dir) {
    process.chdir(dir);
  }

  return _crossSpawn2.default.sync('git', 'symbolic-ref --short -q HEAD'.split(' ')).stdout.toString().trim();
}

/**
 * 判断当前系统是否是macOS
 * @returns {boolean}
 */
function isMac() {
  return _os2.default.type().toLowerCase() === 'darwin';
}

/**
 * 创建临时目录
 * @returns {string}
 */
function createTmpDir() {
  var _getWorkDirs4 = getWorkDirs(),
      TMP_DIR = _getWorkDirs4.TMP_DIR;

  var tmp = _path2.default.join(TMP_DIR, (0, _md2.default)(Date.now()));
  console.log(('\u521B\u5EFA\u4E34\u65F6\u76EE\u5F55\uFF1A' + tmp).green);
  _fs2.default.mkdirSync(tmp);
  return tmp;
}

/**
 * 通过仓库地址获取项目名称
 * @param repository 仓库地址
 * @returns {String} 项目名称
 */
function getProjectNameFromGitUrl(repository) {
  repository = repository || _crossSpawn2.default.sync('git', ['config', '--get', 'remote.origin.url']);
  repository = typeof repository === 'string' ? repository : repository.stdout.toString().trim();
  return repository.split('#')[0].split('/').pop().split('.')[0];
}

/**
 * 获取各个项目文件夹
 * WESHOP_WORK_DIR: weshop 最终 assemble 输出目录
 * PACKAGES_DIR:  weshop 内的各个分包目录
 * DIST_DIR: dist 合包后的目录
 * TMP_DIR: temp 临时目录,
 * WESHOP_DIRS: weshop 相关目录，Array
 * GLOBAL_CONFIG_PATH: weshop 配置文件
 * @returns {{WESHOP_WORK_DIR: *, PACKAGES_DIR: string, DIST_DIR: string, TMP_DIR: string, WESHOP_DIRS: *[], GLOBAL_CONFIG_PATH: string}}
 */
function getWorkDirs() {
  var PROJECT_CONFIG = _fs2.default.existsSync(_constant.PROJECT_CONFIG_FILE_NAME) ? _fsExtra2.default.readJsonSync(_constant.PROJECT_CONFIG_FILE_NAME) : null;
  var WESHOP_CONFIG = _fs2.default.existsSync(_constant.WESHOP_CONFIG_FILE_NAME) ? _fsExtra2.default.readJsonSync(_constant.WESHOP_CONFIG_FILE_NAME) : null;

  if (!PROJECT_CONFIG && !WESHOP_CONFIG) {
    throw new ReferenceError('\u627E\u4E0D\u5230\u5FC5\u987B\u7684\u6587\u4EF6\uFF1Aproject.config.json'.red);
  }

  var env = getParam('--env') || 'dev';
  var appid = WESHOP_CONFIG.env[env] && WESHOP_CONFIG.env[env].appid || PROJECT_CONFIG.appid;

  var WESHOP_HOME = _path2.default.join(_os2.default.homedir(), 'weshop');
  var WESHOP_WORK_DIR = _path2.default.join(WESHOP_HOME, appid);
  var PACKAGES_DIR = [WESHOP_WORK_DIR, 'packages'].join(_constant.SEP);
  var DIST_DIR = [WESHOP_WORK_DIR, 'dist'].join(_constant.SEP);
  var TMP_DIR = [WESHOP_WORK_DIR, 'tmp'].join(_constant.SEP);
  var ASSEMBLE_LOG_DIR = [WESHOP_WORK_DIR, 'log'].join(_constant.SEP);
  var WESHOP_DIRS = [PACKAGES_DIR, DIST_DIR, TMP_DIR];
  var GLOBAL_CONFIG_PATH = [WESHOP_WORK_DIR, 'config.json'].join(_constant.SEP);

  return {
    WESHOP_WORK_DIR: WESHOP_WORK_DIR,
    ASSEMBLE_LOG_DIR: ASSEMBLE_LOG_DIR,
    PACKAGES_DIR: PACKAGES_DIR,
    DIST_DIR: DIST_DIR,
    TMP_DIR: TMP_DIR,
    WESHOP_DIRS: WESHOP_DIRS,
    GLOBAL_CONFIG_PATH: GLOBAL_CONFIG_PATH
  };
}

/**
 * 添加忽略文件
 * @param ignores
 */
function addGitIgnore(ignores, copyRootDir) {
  var PROJECT_ROOT = findProjectRoot();
  var ignorePath = _path2.default.resolve(PROJECT_ROOT, '.gitignore'),
      gitIgnore = void 0;

  _fs2.default.existsSync(ignorePath) || _fs2.default.writeFileSync(ignorePath, '');

  gitIgnore = _fs2.default.readFileSync(ignorePath);

  ignores = Array.isArray(ignores) ? ignores : [ignores];
  ignores.forEach(function (dir) {
    if (copyRootDir) {
      dir = dir.replace(new RegExp('^' + copyRootDir + '/'), '');
    }
    dir = dir.replace(/\\/g, '\/');
    if (gitIgnore.indexOf(dir) < 0) {
      console.log(('\u6DFB\u52A0git\u5FFD\u7565' + (_path2.default.extname(dir) ? '文件' : '目录')).green, dir.blue);
      _crossSpawn2.default.sync('echo', [_constant.EOL, '>>', ignorePath]);
      _crossSpawn2.default.sync('echo', [dir, '>>', ignorePath], { shell: true });
    }
  });
}

/**
 * 判断当前是否主项目
 * @returns {boolean}
 */
function isMainPackage() {
  // 检测项目类型
  var repository = _crossSpawn2.default.sync('git', ['config', '--get', 'remote.origin.url']);
  repository = repository && repository.stdout.toString().trim();

  return repository === getLocalConfig().mainPackage.trim().split('#')[0];
}

/**
 * 解析git操作命令
 * @param args
 * @param options
 */
function gitChildProcessParser(args, options) {
  var cp = _crossSpawn2.default.sync('git', args, options);
  var stdout = cp.stdout.toString();
  var stderr = cp.stderr.toString();
  var repository = void 0;

  if (args[0] === 'clone') {
    repository = args[1];
  } else {
    repository = _crossSpawn2.default.sync('git', ['config', '--get', 'remote.origin.url']).stdout.toString().trim();
  }

  var _parseGitUrl2 = parseGitUrl(repository),
      packageName = _parseGitUrl2.packageName;

  console.log(stdout);
  console.log(stderr);
  if (stderr.indexOf('ssh_exchange_identification') > -1 || stderr.indexOf('Repository does not exist or you do not have access') > -1) {
    throw new URIError('\n      \u65E0\u6CD5\u8BBF\u95EE ' + repository + ' \u53EF\u80FD\u7684\u539F\u56E0\uFF1A\n       1. \u4ED3\u5E93\u662F\u4E0D\u5B58\u5728\n       2. \u6CA1\u6709\u4ED3\u5E93\u6743\u9650\n       3. \u7F51\u7EDC\u4E0D\u7545\u901A\n    ');
  } else if (stderr.indexOf('Permission denied') > -1) {
    throw new Error('无法访问'.red + ' ' + repository.blue + ' ' + '没有权限'.red);
  } else if (stderr.indexOf('Network is down') > -1) {
    throw new Error('\u7F51\u7EDC\u65AD\u5F00\uFF0C\u8BF7\u786E\u8BA4\u7F51\u7EDC\u662F\u5426\u7545\u901A'.red);
  } else if (stderr.indexOf('error: Sparse checkout leaves no entry on working directory') > -1) {
    throw new Error('\u7A00\u758F\u83B7\u53D6\u8DEF\u5F84\u914D\u7F6E\u9519\u8BEF\uFF0C\u65E0\u6CD5\u8FDB\u884C\u7A00\u758F\u62C9\u53D6');
  } else if (stderr.indexOf('did not match any file(s) known to git') > -1) {
    throw new URIError(packageName + ' \u9879\u76EE\u4E0D\u5B58\u5728 ' + (args[1] || 'master') + ' \u5206\u652F\uFF01');
  } else if (stderr.indexOf('Couldn\'t find remote ref') > -1) {
    throw new URIError(packageName + ' \u9879\u76EE\u4E0D\u5B58\u5728 ' + (args[2] || 'master') + ' \u5206\u652F\uFF01');
  }
}

/**
 * 输出信息块标题
 * @param caption
 */
function logStepCaption(caption) {
  console.log(_constant.EOL);
  console.log(''.padEnd(120, '-').green);
  console.log(('>> ' + caption).green);
  console.log(''.padEnd(120, '-').green);
  console.log(_constant.EOL);
}

/**
 * 判断文件是否公共模块
 * @param pth
 */
function isCommon(pth) {
  var PROJECT_ROOT = findProjectRoot();
  pth = _path2.default.isAbsolute(pth) ? _path2.default.relative(PROJECT_ROOT, pth) : pth;
  var is = false;
  getLocalDepPaths(function (ldp) {
    ldp = _path2.default.relative(PROJECT_ROOT, ldp).replace(/\\/g, '\/');
    if (pth.indexOf(ldp) > -1) {
      is = true;
      return false;
    }
  });
  return is;
}

function getGitLastCommit(src) {
  var cwd = process.cwd();
  var commit = void 0;
  process.chdir(src);
  commit = _crossSpawn2.default.sync('git', 'rev-parse HEAD'.split(' '), { shell: true }).stdout.toString().trim();
  process.chdir(cwd);
  return commit;
}

function getGitCurrentBranch(src) {
  var cwd = process.cwd();
  var branch = void 0;
  process.chdir(src);
  branch = _crossSpawn2.default.sync('git', 'rev-parse --abbrev-ref HEAD'.split(' '), { shell: true }).stdout.toString().trim();
  process.chdir(cwd);
  return branch;
}

function getGitLastAuthor(src) {
  var cwd = process.cwd();
  var author = void 0;
  process.chdir(src);
  author = _crossSpawn2.default.sync('git', 'log -1 --pretty=format:\'%an\''.split(' '), { shell: true }).stdout.toString();
  process.chdir(cwd);
  return author;
}

function exportAssembleReport(report) {
  var _getWorkDirs5 = getWorkDirs(),
      ASSEMBLE_LOG_DIR = _getWorkDirs5.ASSEMBLE_LOG_DIR;
  // 生成报告文件

  try {
    _fs2.default.writeFileSync(_path2.default.join(process.cwd(), _constant.ASSEMBLE_REPORT_NAME), JSON.stringify(report, null, _constant.INDENT));

    // 创建 LOG 目录，并根据时间写入 log
    console.log('\u6B63\u5728\u751F\u6210\u62A5\u544A\u6587\u4EF6\u5E76\u4FDD\u5B58\uFF1A' + ASSEMBLE_LOG_DIR);

    if (!_fs2.default.existsSync(ASSEMBLE_LOG_DIR)) {
      _fs2.default.mkdirSync(ASSEMBLE_LOG_DIR);
    }
    var filename = (0, _moment2.default)().format('YYYYMMDD_HHmmss') + '.json';
    _fs2.default.writeFileSync(_path2.default.join(ASSEMBLE_LOG_DIR, filename), JSON.stringify(report, null, _constant.INDENT));

    if (_fs2.default.existsSync('.git')) {
      addGitIgnore(_constant.ASSEMBLE_REPORT_NAME);
    }

    return filename;
  } catch (e) {
    console.log(e);
  }
}

function getParam(key) {
  var argumentsArr = process.argv;

  // 单参数
  if (/^(-){1}\w+$/.test(key)) {
    return argumentsArr.includes(key);
  }

  // 多参数
  if (/^(-){2}\w+$/.test(key)) {
    var keyIndex = argumentsArr.indexOf(key);

    if (keyIndex === -1) {
      return undefined;
    } else {
      return argumentsArr[keyIndex + 1];
    }
  }

  return undefined;
}

/**
 * 对 packages 文件夹下的仓库打上标签
 * @param {String} tagName 标签名
 */
function buildTag(_ref, logName) {
  var tag = _ref.tag,
      message = _ref.message;

  var _getWorkDirs6 = getWorkDirs(),
      ASSEMBLE_LOG_DIR = _getWorkDirs6.ASSEMBLE_LOG_DIR,
      PACKAGES_DIR = _getWorkDirs6.PACKAGES_DIR;

  var logFile = _path2.default.join(ASSEMBLE_LOG_DIR, logName);
  var logContent = _fsExtra2.default.readJsonSync(logFile);
  var packages = Object.keys(logContent);

  packages.forEach(function (project) {
    if (project === 'dgd-weui') return false;

    var packagePath = _path2.default.join(PACKAGES_DIR, project);
    process.chdir(packagePath);

    (0, _child_process.spawnSync)('git', ['tag', '-a', tag, '-m', message]);
    (0, _child_process.spawnSync)('git', ['push', 'origin', tag]);
  });

  process.chdir(process.cwd());
  console.log(('\u5DF2\u6253\u4E0A Tag \u6807\u7B7E\uFF1A' + tag).green);
}